! Time-stamp: <wfe1.fsl.noaa.gov:/home/jacobson/TM5/cy2/proj/chem/enkf/base/src/emission_co2_bio.F90: 31 Jul 2008 (Thu) 17:19:20 GMT>

!### macros #####################################################
!
#define IF_NOTOK_RETURN(action) if (status/=0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; write (gol,'("in ",a)') rname; call goErr; action; return; end if
!
!#################################################################

module emission_co2_bio
  !-----------------------------------------------------------------------
  !       purpose
  !       -------
  !       perform co2 biosphere emissions 
  !
  !       EMISSIONS
  !          interface:     call emission_apply_bio
  !          method:        subroutine is called from its general parent routine apply_emission
  !
  !       COVARIANCE
  !          interface:     call covariance_co2_compute_bio
  !          method:        subroutine is called from MakePriorUncertainty in enkf_tools.F90
  !
  ! Table of contents:
  !   module emission_co2_bio
  !     subroutine declare_emission_co2_bio
  !     end subroutine declare_emission_co2_bio
  !     subroutine calc_emission_co2_bio
  !     end subroutine calc_emission_co2_bio
  !     subroutine emission_apply_co2_bio(region)
  !     end subroutine emission_apply_co2_bio
  !     subroutine free_emission_co2_bio
  !     end subroutine free_emission_co2_bio
  !     subroutine covariance_co2_compute_bio(Q,parm_start,parm_end)
  !     end subroutine covariance_co2_compute_bio
  !   end module emission_co2_bio
  !
  !-----------------------------------------------------------------------
  
  use GO,                  only: gol, goErr, goPr, goLabel
  use Dims,                only: nregions, nlon360,nlat180
  use Meteo,               only: t2m_sfc,ssr_sfc
  use Meteo,               only: Set
  use enkf_common,         only: scalefac    ! current indirect access to parameters
  use enkf_common,         only: state_X     ! for future direct use of parameter values
  use partools,           only: nmembersloc ! to interpret state_X, scalefac, nmembersloc
  use global_types,        only: emis_data
  use emission_data,       only: do_add_2d, flux_to_gridbox_per_month
  use emission_common,     only: enkf_is_forward,enkf_is_inverse, EmisInputDir
  use emission_common,     only: co2_bio, co2_bio_m, co2_bio_n, flux_means_are_priors

  implicit none

  ! --- const -------------------------
  character(len=*), parameter  ::  mname = 'emission_co2_bio'

  private

  ! public routines:
  public   :: declare_emission_co2_bio, free_emission_co2_bio, calc_emission_co2_bio, emission_apply_co2_bio
  public   :: covariance_co2_compute_bio

  logical                                :: use_fires

  ! co2_bio_regxmem is the result of the calculate routine.
  ! It holds the coarsened flux per ensemble member and region.
  type(emis_data),dimension(:,:),allocatable,target       :: co2_bio_regxmem

contains

  subroutine declare_emission_co2_bio
    use GO,                  only : TrcFile, Init, Done, ReadRc
    use global_data,         only : rcfile
    use dims,       only: im,jm,nregions

    implicit none
    integer                                          :: region, imr, jmr, n
    integer                                      :: status
    type(TrcFile)        :: rcF

    character(len=*), parameter        :: rname = mname//'/declare_emission_co2_bio'

    call Init( rcF, rcfile, status )
    call ReadRc( rcF, 'emis.use.fires', use_fires ,status ,default=.false.)
    call Done( rcF, status )

    call Set(   t2m_sfc, status, used=.true. )
    call Set(   ssr_sfc, status, used=.true. )

    if(enkf_is_forward()) then
       allocate(co2_bio_regxmem(nregions,nmembersloc))

       do region=1,nregions
          imr=im(region)
          jmr=jm(region)
          do n=1,nmembersloc
             allocate(co2_bio_regxmem(region,n)%surf(imr,jmr))
          end do
       enddo
    end if

  end subroutine declare_emission_co2_bio

  subroutine calc_emission_co2_bio
    use dims,    only        : nlon360, nlat180, im,jm, sec_month, newsrun, idate, itau, staggered, nread, newmonth
    use io_save, only        : ReadFromFile
    use toolbox, only        : escape_tm
    use toolbox, only        : coarsen_emission_2d
    use emission_data, only  : chardate
    use Meteo, only          : Set
    use chem_param, only     : xmc
    use go_string, only      : goNum2str
    implicit none
    integer                                      :: status
    integer                                      :: region, imr, jmr, mem
    integer, parameter                           :: annual_field = 0, rank2 = 2, level1 = 1
    real,dimension(nlon360,nlat180)              :: p0,r0
    real,dimension(nlon360,nlat180)              :: co2_bio_mem ! flux for a given ensemble member
    integer, parameter                           :: add_field=0

    real,pointer,dimension(:,:)                  :: t2m,ssr
    character(len=10)                            :: cdate

    character(len=*), parameter        :: rname = mname//'/calc_emission_co2_bio'

    !WP! CAUTION, this statement will prevent accumulation of flux1x1 means on timescale that are not integer multiples of 
    !WP! nread. Perhaps in the future change this to hourly, or every time step for instance when interoplation of meteorology
    !WP! is used???!

    if(mod(itau,nread) /= 0) return  ! only every nread hours

    if(newmonth) then
        cdate=trim(goNum2str(idate(1),'(i4.4)'))//trim(goNum2str(idate(2),'(i2.2)'))
       if(use_fires) then
          call ReadFromFile(trim(EmisInputDir)//'/biofireparams_'//trim(cdate)//'.hdf','rparam1',r0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
          call ReadFromFile(trim(EmisInputDir)//'/biofireparams_'//trim(cdate)//'.hdf','pparam1',p0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
       else 
          call ReadFromFile(trim(EmisInputDir)//'/bioparams_'//trim(cdate)//'.hdf','rparam1',r0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
          call ReadFromFile(trim(EmisInputDir)//'/bioparams_'//trim(cdate)//'.hdf','pparam1',p0,status, sds_date=(/idate(1),idate(2),1,0,0,0/))
       endif
       if (status /= 0) call escape_tm('stop in '//rname)
    endif

    t2m=> t2m_sfc%data(:,:,1)
    ssr=> ssr_sfc%data(:,:,1)

    co2_bio=r0*(1.5**((t2m-273.15)/10.0))  ! Q10 relation for repsiration
    co2_bio=co2_bio+p0*ssr        ! radiation scaling for GPP 

    nullify(t2m)
    nullify(ssr)

    ! accumulate weekly average
    if(allocated(co2_bio_m)) then
       if(flux_means_are_priors .eq. .TRUE.) then
          co2_bio_m=co2_bio_m+co2_bio ! mol/m2/s
       else
          co2_bio_m=co2_bio_m+co2_bio*scalefac(:,:,1) ! mol/m2/s, tracer 0 on PE0 is mean
       endif
       co2_bio_n=co2_bio_n+1
    endif

    ! The rest only useful to the forward code
    if(enkf_is_inverse()) return

    ! Apply net flux scaling by ensemble member, change units, and
    ! coarsen to the regions.  When parameters are used beyond simple
    ! scaling of the net flux, some piece of logic from above will
    ! have to be subsumed by this loop to get the parameter
    ! deviations.

    do mem=1,nmembersloc
       co2_bio_mem=scalefac(:,:,mem)*co2_bio ! parameter applied, mol/m2/s
       co2_bio_mem=co2_bio_mem*xmc/1.e3 ! from mol/m2/s to kgC/m2/s
       call flux_to_gridbox_per_month(co2_bio_mem) ! to kgC/area/month
       call coarsen_emission_2d('co2_bio',360,180,co2_bio_mem,co2_bio_regxmem(:,mem),add_field)
    enddo

  end subroutine calc_emission_co2_bio

  subroutine emission_apply_co2_bio(region)

    ! This routine should only be called by the forward enkf code.  It
    ! applies the bio co2 flux by member and region to the ico2bio
    ! tracer.  In the inverse code, the calculated flux from each
    ! module is applied to the (single) co2 tracer.

    use dims,          only: okdebug
    use ParTools,      only: tracer_active
    use chem_param, only         : ico2bio, xmc, xmco2


    implicit none

    integer, intent(in)        :: region
    integer                    :: mem

    character(len=*), parameter        :: rname = mname//'/emission_apply_co2_bio'

    if(enkf_is_forward()) then
       if(ico2bio .gt. 0) then
          do mem=1,nmembersloc
             call do_add_2d(region,ico2bio,1,co2_bio_regxmem(region,mem)%surf,xmco2,xmc)
          enddo
       endif
    endif

    if(okdebug) write(*,*) 'end of emission_apply_co2_bio'

  end subroutine emission_apply_co2_bio

  subroutine free_emission_co2_bio
    use dims,        only: nregions

    implicit none
    integer               :: region,n

    character(len=*), parameter        :: rname = mname//'/free_emission_co2_bio'

    if(enkf_is_forward()) then
       do region=1,nregions
          do n=1,nmembersloc
             deallocate(co2_bio_regxmem(region,n)%surf)
          end do
       enddo
       
       deallocate(co2_bio_regxmem)
    endif

  end subroutine free_emission_co2_bio

  subroutine covariance_co2_compute_bio(Q,parm_start,parm_end)

    use GO,           only : gol, goErr, goPr, goLabel
    use GO,           only : TrcFile, Init, Done, ReadRc
    use global_data,  only : rcfile
    use io_save,      only : ReadFromFile
    use io_hdf,       only : io_write
    use io_hdf,       only : DFNT_INT32, DFNT_CHAR, DFNT_FLOAT64, DFACC_CREATE, DFACC_WRITE
    use go_string,    only : goNum2str

    implicit none

    integer                            :: parm_start,parm_end
    real,dimension(:,:)                :: Q

    type(TrcFile)                      :: rcF
    integer                            :: nparms,i,j,status
    character(len=256)                 :: covariance_file
    real,dimension(:,:),allocatable    :: bio_priorcov

    character(len=*), parameter        :: rname = mname//'/covariance_co2_compute_bio'

    call Init( rcF, rcfile, status)
    IF_NOTOK_RETURN(status=1)
    call ReadRc( rcF, 'bio.covariance', covariance_file, status)
    IF_NOTOK_RETURN(status=1)
    call Done( rcF, status )
    IF_NOTOK_RETURN(status=1)

    nparms = parm_end-parm_start+1

    allocate(bio_priorcov(nparms,nparms))

    call ReadFromFile(covariance_file,'qprior',bio_priorcov,status)

    Q(parm_start:parm_end, parm_start:parm_end) = bio_priorcov

    deallocate(bio_priorcov)

  end subroutine covariance_co2_compute_bio


end module emission_co2_bio
