!#################################################################
!
! tracer and air mass
!
!### macro's #####################################################
!
#define TRACEBACK write (gol,'("in ",a," (",a,", line",i5,")")') rname, __FILE__, __LINE__; call goErr
#define IF_NOTOK_RETURN(action) if (status/=0) then; TRACEBACK; action; return; end if
#define IF_ERROR_RETURN(action) if (status> 0) then; TRACEBACK; action; return; end if
!
!#################################################################
  
  
module tracer_data

! define preprocessing variable 'secmom' if necessary:
#include "advect.h"

  use GO  , only : gol, goErr, goPr
  use dims, only : nregions
  use global_types, only : chem_data

  implicit none
  
  ! --- in/out ------------------------------------
  
  private
  
  public  ::  mass_data, mass_dat
  public  ::  chem_dat
  public  ::  assign_massdat, free_massdat

  public  ::  Par_Check_Domain
  public  ::  Par_Check_Mass

  public  ::  AdjustTracer
  
  public :: tracer_print
  

  ! --- const --------------------------------------

  character(len=*), parameter  ::  mname = 'tracer_data'


  ! --- types ------------------------------------

  type mass_data

     ! m_k     air mass (kg)   (halo = 2)
     ! p       surface pressure (Pa)   (halo = 2)
     ! phlb_k  pressure at the grid boundaries (Pa) : 
     !         l=1 means bottom lowest box (halo = 0)
     ! rm_k    tracer masses (kg)    (halo = 2)
     ! rxm_k   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     ! rym_k   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     ! rzm_k   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     ! m_t     air mass (kg)   (halo = 2)
     ! phlb_t  pressure at the grid boundaries (Pa) : 
     !         l=1 means bottom lowest box (halo = 0)
     ! rm_t    tracer masses (kg)    (halo = 2)
     ! rxm_t   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     ! rym_t   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     ! rzm_t   tracer slopes (only transported) in kg/(halfgridsize) (halo = 1)
     !
     real,dimension(:,:,:),  pointer   :: m_k 
     real,dimension(:,:,:),  pointer   :: phlb_k
     real,dimension(:,:,:,:),pointer   :: rm_k
     real,dimension(:,:,:,:),pointer   :: rxm_k
     real,dimension(:,:,:,:),pointer   :: rym_k
     real,dimension(:,:,:,:),pointer   :: rzm_k
#ifdef secmom
     real,dimension(:,:,:,:),pointer   :: rxxm_k
     real,dimension(:,:,:,:),pointer   :: rxym_k
     real,dimension(:,:,:,:),pointer   :: rxzm_k
     real,dimension(:,:,:,:),pointer   :: ryym_k
     real,dimension(:,:,:,:),pointer   :: ryzm_k
     real,dimension(:,:,:,:),pointer   :: rzzm_k
#endif
     real,dimension(:,:,:),  pointer   :: m_t 
     real,dimension(:,:,:),  pointer   :: phlb_t
     real,dimension(:,:,:,:),pointer   :: rm_t 
     real,dimension(:,:,:,:),pointer   :: rxm_t
     real,dimension(:,:,:,:),pointer   :: rym_t
     real,dimension(:,:,:,:),pointer   :: rzm_t
#ifdef secmom
     real,dimension(:,:,:,:),pointer   :: rxxm_t
     real,dimension(:,:,:,:),pointer   :: rxym_t
     real,dimension(:,:,:,:),pointer   :: rxzm_t
     real,dimension(:,:,:,:),pointer   :: ryym_t
     real,dimension(:,:,:,:),pointer   :: ryzm_t
     real,dimension(:,:,:,:),pointer   :: rzzm_t
#endif

  end type mass_data


  ! --- var ------------------------------

  type(mass_data), target   :: mass_dat(nregions)
  type(chem_data), target   :: chem_dat(nregions)



contains


  ! ======================================================================


  subroutine assign_massdat( reg, iaction )

    use dims      , only : nregions
    use dims      , only : im, jm, lm
    use chem_param, only : ntracet
    use partools  , only : ntracetloc, lmloc
    
    ! --- in/out -------------------------

    integer, intent(in)             ::  reg
    integer, intent(in)             ::  iaction
    
    ! --- local -------------------------

    integer            ::   imr, jmr, lmr, nt
    
    ! --- begin -------------------------

    select case ( iaction )
    
      !WP! swapping from tracer to k
      
      case ( 0 )
      
#ifdef MPI      
       imr = im(reg)
       jmr = jm(reg)
       lmr = lmloc
       nt  = ntracet
       allocate ( mass_dat(reg)%m_k  (-1:imr+2,-1:jmr+2,lmr) ) 
       allocate ( mass_dat(reg)%rm_k (-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rym_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rzm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
#ifdef secmom
       allocate ( mass_dat(reg)%rxxm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxym_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxzm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%ryym_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%ryzm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rzzm_k(-1:imr+2,-1:jmr+2,lmr,nt) )
#endif
       mass_dat(reg)%m_k  =0.0
       mass_dat(reg)%rm_k =0.0
       mass_dat(reg)%rxm_k=0.0
       mass_dat(reg)%rym_k=0.0
       mass_dat(reg)%rzm_k=0.0
#ifdef secmom
       mass_dat(reg)%rxxm_k=0.0
       mass_dat(reg)%rxym_k=0.0
       mass_dat(reg)%rxzm_k=0.0
       mass_dat(reg)%ryym_k=0.0
       mass_dat(reg)%ryzm_k=0.0
       mass_dat(reg)%rzzm_k=0.0
#endif
#endif

    
    ! swapping from k to tracer

    case ( 1 )
    
       imr = im(reg)
       jmr = jm(reg)
       lmr = lm(reg) 
       nt  = ntracetloc

       allocate ( mass_dat(reg)%m_t  (-1:imr+2,-1:jmr+2,lmr)    ) 
       allocate ( mass_dat(reg)%rm_t (-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rym_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rzm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
#ifdef secmom
       allocate ( mass_dat(reg)%rxxm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxym_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rxzm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%ryym_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%ryzm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
       allocate ( mass_dat(reg)%rzzm_t(-1:imr+2,-1:jmr+2,lmr,nt) )
#endif
       mass_dat(reg)%m_t=0.0
       mass_dat(reg)%rm_t=0.0
       mass_dat(reg)%rxm_t=0.0
       mass_dat(reg)%rym_t=0.0
       mass_dat(reg)%rzm_t=0.0
#ifdef secmom
       mass_dat(reg)%rxxm_t=0.0
       mass_dat(reg)%rxym_t=0.0
       mass_dat(reg)%rxzm_t=0.0
       mass_dat(reg)%ryym_t=0.0
       mass_dat(reg)%ryzm_t=0.0
       mass_dat(reg)%rzzm_t=0.0
#endif
  
    end select

  end subroutine assign_massdat


  ! ***
  

  subroutine free_massdat( reg, iaction )

    use dims      , only : nregions

    ! --- in/out -------------------------

    integer, intent(in)             ::  reg
    integer, intent(in)             ::  iaction
    
    ! --- local -------------------------

    integer            ::   imr, jmr, lmr, nt
    
    ! --- begin -------------------------

    select case ( iaction )
    
      ! swapping from tracer to k
      case ( 0 )
      
        deallocate ( mass_dat(reg)%m_t)
        deallocate ( mass_dat(reg)%rm_t)
        deallocate ( mass_dat(reg)%rxm_t)
        deallocate ( mass_dat(reg)%rym_t)
        deallocate ( mass_dat(reg)%rzm_t)
#ifdef secmom
        deallocate ( mass_dat(reg)%rxxm_t)
        deallocate ( mass_dat(reg)%rxym_t)
        deallocate ( mass_dat(reg)%rxzm_t)
        deallocate ( mass_dat(reg)%ryym_t)
        deallocate ( mass_dat(reg)%ryzm_t)
        deallocate ( mass_dat(reg)%rzzm_t)
#endif

      !WP! swapping from k to tracer 
      case ( 1 )
    
#ifdef MPI      
        deallocate ( mass_dat(reg)%m_k)
        deallocate ( mass_dat(reg)%rm_k)
        deallocate ( mass_dat(reg)%rxm_k)
        deallocate ( mass_dat(reg)%rym_k)
        deallocate ( mass_dat(reg)%rzm_k)
#ifdef secmom
        deallocate ( mass_dat(reg)%rxxm_k)
        deallocate ( mass_dat(reg)%rxym_k)
        deallocate ( mass_dat(reg)%rxzm_k)
        deallocate ( mass_dat(reg)%ryym_k)
        deallocate ( mass_dat(reg)%ryzm_k)
        deallocate ( mass_dat(reg)%rzzm_k)
#endif
#endif

    end select

  end subroutine free_massdat
  
  
  ! ===================================================================


  subroutine Par_Check_Domain( region1, others, to_check )

    use dims     , only : parent,children, okdebug
    use ParTools , only : myid, root
    use ParTools , only : previous_par
    use ParTools , only : allocate_mass
#ifdef MPI
    use mpi_comm , only : barrier
    use mpi_comm , only : escape_tm_mpi
#endif

    ! --- in/out -----------------------------------
    
    integer,intent(in)              ::  region1 
    character(len=*),intent(in)     ::  others
    character(len=*),intent(in)     ::  to_check

    ! --- local ------------------------------------
    
#ifdef MPI   
    integer :: reg,regstart,regstop,iaction
    integer :: nchild,regiop
#endif

    ! --- begin ------------------------------------
      
#ifdef MPI   

    select case(others)
      case('p') !WP! also check parents
        regiop=parent(region1)
        if ( regiop == 0 ) then 
          regstart=region1
          regstop=region1
        else if ( regiop /= 0 ) then
          regstart=regiop
          regstop=region1
        end if
      case('c') !WP! also check child  
        nchild=children(region1,0)
        if ( nchild == 0 ) then
          regstart=region1
          regstop=region1
        else if ( nchild /= 0 ) then
          regstart=region1
          !WP! take number of last child as regstop
          regstop=children(region1,nchild)
        end if
      case('n') !WP! check no parents or children
        regstart=region1
        regstop=region1
      case default
        call escape_tm_mpi('check_domain: illegal value of *others* ')
    end select

    select case(to_check)
      case('levels')
        iaction=0
      case('tracer') 
        iaction=1
      case default
        call escape_tm_mpi('check_domain: illegal value of *to_check* ')
    end select

    do reg=regstart,regstop
      ! make sure data is in requested domain
      if ( previous_par(reg) /= to_check ) then
        if ( okdebug ) then
          write (gol,'("check_domain: data will be swapped")'); call goPr
        end if
        previous_par(reg)=to_check   ! reset previous par
        ! assign new memory
        if ( allocate_mass ) call assign_massdat(reg,iaction)
        call barrier
        ! swap data for this region
        call swap_all_mass(reg,iaction)
        call barrier
        ! free previously allocated memory
        if ( allocate_mass ) call free_massdat(reg,iaction)
      else
        if ( okdebug ) then
          write (gol,'("check_domain: data is on proper communicator, nothing to do")'); call goPr
        end if
      end if
    end do

#endif

  end subroutine Par_Check_Domain


  ! ***
  

#ifdef MPI      

  !WP! subroutine swaps the arrays m,rm,rxm,rym,and rzm 
  !WP! from levels to tracers (iaction=1)
  !WP! or from levels to tracers (iaction=0)
  !WP! for requested regions (region1=>region2)    
  !WP! after swapping, old data is set to zero to avoid mistakes 
  !WP! with 'old' data
  !WP! 23 december 2002

  subroutine swap_all_mass(region,iaction)

    use dims,       only : okdebug, im,jm ,lm
    use chem_param, only : ntracet
    use mpi_const , only : myid, root
    use mpi_const , only : ntracetloc, ntracet_ar
    use mpi_comm  , only : barrier
    use mpi_comm  , only : scatter_after_read_k_3d, gather_tracer_k_3d
    use mpi_comm  , only : tracer_to_k, k_to_tracer

    ! --- in/out ----------------------------
    
    integer,intent(in) :: iaction
    integer,intent(in) :: region
    
    ! --- local -----------------------------

    integer :: imr,jmr,lmr
    real,dimension(:,:,:),pointer     :: msource
    real,dimension(:,:,:),pointer     :: mtarget
    real,dimension(:,:,:,:),pointer     :: rxmsource
    real,dimension(:,:,:,:),pointer     :: rxmtarget
    integer :: nsend,xx, xx_end

    ! --- begin -----------------------------
    
    imr=im(region)
    jmr=jm(region)
    lmr=lm(region)

    if ( iaction == 0 ) then

      msource => mass_dat(region)%m_t
      mtarget => mass_dat(region)%m_k
      !WP! the 3d array mass is correct on root_t, and scattered to levels 
      call scatter_after_read_k_3d( msource, imr,jmr,lmr, 2,2,0, &
                                    mtarget, root )
      call barrier                              
      nullify(msource)
      nullify(mtarget)

    else if ( iaction == 1 ) then

      !WP! gather mass to root_k, and broadcast to all
      !WP! only PE's in com_trac need mloc_tracer, 
      !WP! but we cannot guarantee that root_k is 
      !WP! also in com_trac, whcih would lead to errors.

      msource => mass_dat(region)%m_k
      mtarget => mass_dat(region)%m_t

      call  gather_tracer_k_3d( mtarget, imr,jmr,lmr, 2,2,0, &
                                msource, .true. ) !WP! gather and broadcast

      nullify(msource) 
      nullify(mtarget) 

    end if
    if ( okdebug ) then
      write (gol,'("swap_all_mass: Scattered mass")'); call goPr
    end if

#ifdef secmom
    xx_end = 10
#else
    xx_end=4
#endif
        
    !WP! once for each x,y, and z
    do xx = 1, xx_end   
      select case(xx)
        case( 1 )
          rxmsource => mass_dat(region)%rm_t
          rxmtarget => mass_dat(region)%rm_k
        case( 2 )
          rxmsource => mass_dat(region)%rxm_t
          rxmtarget => mass_dat(region)%rxm_k
        case( 3 )
          rxmsource => mass_dat(region)%rym_t
          rxmtarget => mass_dat(region)%rym_k
        case( 4 )
          rxmsource => mass_dat(region)%rzm_t
          rxmtarget => mass_dat(region)%rzm_k
#ifdef secmom
        case( 5 )
          rxmsource => mass_dat(region)%rxxm_t
          rxmtarget => mass_dat(region)%rxxm_k
        case( 6 )
          rxmsource => mass_dat(region)%rxym_t
          rxmtarget => mass_dat(region)%rxym_k
        case( 7 )
          rxmsource => mass_dat(region)%ryym_t
          rxmtarget => mass_dat(region)%ryym_k
        case( 8 )
          rxmsource => mass_dat(region)%rxzm_t
          rxmtarget => mass_dat(region)%rxzm_k
        case( 9 )
          rxmsource => mass_dat(region)%ryzm_t
          rxmtarget => mass_dat(region)%ryzm_k
        case( 10 )
          rxmsource => mass_dat(region)%rzzm_t
          rxmtarget => mass_dat(region)%rzzm_k
#endif
      end select
      if ( iaction == 0 ) then
        call tracer_to_k( rxmtarget, rxmsource, &
                            imr,jmr,lmr, 2,2,0, ntracet,ntracetloc, &
                            ntracet_ar )
        call barrier
      else if ( iaction == 1 ) then
        call k_to_tracer( rxmtarget, rxmsource, &
                            imr,jmr,lmr, 2,2,0, ntracet, ntracetloc, &
                            ntracet_ar)
      end if
      nullify(rxmsource)               
      nullify(rxmtarget)

      if ( okdebug ) then
        write (gol,'("swap_all_mass: Scattered tracer mass and slopes ",i4)') xx; call goPr
      end if

    end do

  end subroutine swap_all_mass

#endif


  ! ================================================================
  

  subroutine Par_Check_Mass( region, text )

#ifdef MPI
    use dims      , only : lm
    use chem_param, only : ntracet
    use mpi_const , only : myid, root_t, root_k
    use mpi_const , only : ntracetloc, lmloc, previous_par
    use mpi_comm  , only : barrier
    use mpi_comm  , only : dump_field3d, dump_field4d
    use mpi_comm  , only : summasr
#endif

    ! --- in/out -----------------------------------
    
    integer, intent(in)             ::  region
    character(len=*), intent(in)    ::  text

    ! --- local ------------------------------------
    
#ifdef MPI
    real,dimension(:,:,:),pointer     :: m
    real,dimension(:,:,:,:),pointer   :: rm
    integer :: lmr
#endif

    ! --- begin ------------------------------------

#ifdef MPI

    call barrier   
    if ( previous_par(region) == 'tracer' ) then
       if ( myid == root_t ) print*,'check_mass: checking mass in ',text
       call barrier
       m => mass_dat(region)%m_t
       rm => mass_dat(region)%rm_t
       lmr=lm(region)

       call dump_field3d(region,m,lmr,(/2,2,0/),'m','mtrac.hdf')
       call dump_field4d(region,rm,lmr,ntracetloc,(/2,2,0/),'rm','rmtrac.hdf')
       call summasr( region, m, rm )
    else if ( previous_par(region) == 'levels' ) then   
       if ( myid == root_k ) print*,'check_mass: checking mass in ',text
       call barrier
       m => mass_dat(region)%m_k
       rm => mass_dat(region)%rm_k
       lmr=lmloc

       call dump_field3d(region,m,lmr,(/2,2,0/),'m','mlev.hdf')
       call dump_field4d(region,rm,lmr,ntracet,(/2,2,0/),'rm','rmlev.hdf')
       call summasr( region, m, rm )
    end if
    call barrier
    nullify(m)
    nullify(rm)

#endif

  end subroutine Par_Check_Mass



  ! ================================================================
  
    
  !
  ! Tracer slopes (kg/half cell) and second moments (kg/(half cell)^2)
  ! are adjusted with the same factor as applied to the total mass
  ! (background: if the concentration 'in the middle' is decreased
  ! by a factor 0.5, then the concentrations at the boundaries are
  ! also changed with a factor 0.5 :
  !
  !       ^      |       |             |       |
  ! tracer|    6 |       o             |       |
  !  mass |      |     - |             |       |
  !  (kg)      4 |   o   |     ->      |       |    
  !              | -     |           3 |     _ o
  !            2 o       |           2 | _ o   |
  !              |       |           1 o       |
  !           ---------------       ---------------
  !
  ! NOTE: not implemented for parallel yet ...
  !  ? should l be a local or a global level ?
  !  ? assume that tracer k is on this pe ?
  !

  subroutine AdjustTracer( drm, region, i, j, l, k, status )

    use GO      , only : gol, goErr  
    use dims    , only : adv_scheme
    use ParTools, only : which_par, tracer_active, lmloc

    ! --- in/out -----------------------------  
    
    real, intent(in)           ::  drm           ! tracer mass change (kg)
    integer, intent(in)        ::  region
    integer, intent(in)        ::  i, j           ! cell indices
    integer, intent(in)        ::  l              ! local level
    integer, intent(in)        ::  k             ! tracer index
    integer, intent(out)       ::  status
    
    ! --- const -------------------------------
    
    character(len=*), parameter  ::  rname = mname//'/AdjustTracer'
    
    ! --- local -------------------------------
    
    real, dimension(:,:,:,:), pointer   :: rm
    real, dimension(:,:,:,:), pointer   :: rxm, rym, rzm
#ifdef secmom
    real, dimension(:,:,:,:), pointer   :: rxxm, rxym, rxzm, ryym, ryzm, rzzm
#endif

    real   ::  rm_old
    real   ::  skf

    ! --- begin -----------------------------
    
    ! set pointers
    select case ( which_par )
      case ( 'levels' )
        ! check local level index:
        if ( l > lmloc ) then
          write (gol,'("found strange local level:")'); call goErr
          write (gol,'("  l, lmloc  :  ", 2i4)') l, lmloc; call goErr
          write (gol,'("in ",a)') rname; call goErr; status=1; return
        end if
        ! set pointers:
        rm   => mass_dat(region)%rm_k
        rxm  => mass_dat(region)%rxm_k
        rym  => mass_dat(region)%rym_k
        rzm  => mass_dat(region)%rzm_k
#ifdef secmom
        rxxm => mass_dat(region)%rxxm_k
        rxym => mass_dat(region)%rxym_k
        rxzm => mass_dat(region)%rxzm_k
        ryym => mass_dat(region)%ryym_k
        ryzm => mass_dat(region)%ryzm_k
        rzzm => mass_dat(region)%rzzm_k
#endif
      case ( 'tracer' )
        ! leave if this pe does not contain this tracer :
        if ( .not. tracer_active(k) ) then
          status=0; return
        end if
        ! set pointers:
        rm   => mass_dat(region)%rm_t
        rxm  => mass_dat(region)%rxm_t
        rym  => mass_dat(region)%rym_t
        rzm  => mass_dat(region)%rzm_t
#ifdef secmom
        rxxm => mass_dat(region)%rxxm_t
        rxym => mass_dat(region)%rxym_t
        rxzm => mass_dat(region)%rxzm_t
        ryym => mass_dat(region)%ryym_t
        ryzm => mass_dat(region)%ryzm_t
        rzzm => mass_dat(region)%rzzm_t
#endif
      case default
        write (gol,'("unsupported which_par : ",a)') trim(which_par); call goErr
        write (gol,'("in ",a)') rname; call goErr; status=1; return
    end select

    ! store old tracer mass:
    rm_old = rm(i,j,l,k)
    
    ! store new tracer mass:
    rm(i,j,l,k) = rm_old + drm
    
    ! adjust concentration gradients ?
    if ( (adv_scheme == 'slope') .or. (adv_scheme == '2nd_m') ) then

      ! scale factor to be applied; trap devision by zero:
      if ( (rm(i,j,l,k) > tiny(1.0)) .and. (rm_old > tiny(0.0)) ) then
        skf = rm(i,j,l,k) / rm_old 
      else
        skf = 0.0
      end if

      ! adjust slopes:
      rxm(i,j,l,k) = rxm(i,j,l,k) * skf
      rym(i,j,l,k) = rym(i,j,l,k) * skf
      rzm(i,j,l,k) = rzm(i,j,l,k) * skf

#ifdef secmom
      ! adjust second moments ?
      if ( adv_scheme == '2nd_m' ) then
        rxxm(i,j,l,k) = rxxm(i,j,l,k) * skf
        rxym(i,j,l,k) = rxym(i,j,l,k) * skf
        rxzm(i,j,l,k) = rxzm(i,j,l,k) * skf
        ryym(i,j,l,k) = ryym(i,j,l,k) * skf
        ryzm(i,j,l,k) = ryzm(i,j,l,k) * skf
        rzzm(i,j,l,k) = rzzm(i,j,l,k) * skf
      end if
#endif

    end if  ! adjust gradients
    
    ! ok
    status = 0
    
  end subroutine AdjustTracer
  

  ! ***
  
  subroutine tracer_print( key, region, status )
  
    use GO
    use dims, only : im, jm, lm
    use chem_param, only : ntrace, ntracet, names
    !use chem_param, only : io3
    use partools, only : myid
    use partools, only : lmar, lmloc, tracer_active, tracer_loc
    use partools, only : previous_par, which_par
    use partools, only : par_barrier
    !use sources_sinks, only : chem_dat
  
    ! --- in/out ---------------------------------
    
    character(len=*), intent(in)  ::  key
    integer, intent(in)           ::  region
    integer, intent(out)          ::  status
    
    ! --- const ------------------------------

    character(len=*), parameter ::  rname = mname//'/tracer_print'
    
    ! --- local ---------------------------------
    
    integer             ::  i, j
    integer             ::  l, l_base, l_loc
    integer             ::  imr, jmr, lmr
    real, allocatable   ::  pat(:,:)
    real, allocatable   ::  patx(:,:)
    real, allocatable   ::  paty(:,:)
    real, allocatable   ::  patz(:,:)
    integer             ::  itr
    logical             ::  filled
    
    ! --- begin ----------------------------------
    
    imr = im(region)
    jmr = jm(region)
    lmr = lm(region)
    
    allocate( pat(imr,jmr) )
    allocate( patx(imr,jmr) )
    allocate( paty(imr,jmr) )
    allocate( patz(imr,jmr) )

    if (tracer_active(1)) then    
    write (gol,'(a," region, previous_par : ",i1," ",a)') key, region, previous_par(region); call goBug
    endif
    call par_barrier

    do itr = 1, ntrace

      !if (itr/=io3) cycle
      !if (itr/=1) cycle

      do l = 1, lmr   
      
        if (l/=1) cycle 

        filled = .false.
        if ( itr <= ntracet ) then
          select case ( previous_par(region) )
            case ( 'levels' )
              l_base = 0
              if (myid>0) l_base = sum(lmar(0:myid-1))
              l_loc = l - l_base
              if ( (l_loc >= 1) .and. (l_loc <= lmloc) ) then
                pat  = mass_dat(region)%rm_k (1:imr,1:jmr,l_loc,itr)
                patx = mass_dat(region)%rxm_k(1:imr,1:jmr,l_loc,itr)
                paty = mass_dat(region)%rym_k(1:imr,1:jmr,l_loc,itr)
                patz = mass_dat(region)%rzm_k(1:imr,1:jmr,l_loc,itr)
                filled = .true.
              end if
            case ( 'tracer' )
              if ( tracer_active(itr) ) then
                pat  = mass_dat(region)%rm_t (1:imr,1:jmr,l,tracer_loc(itr))
                patx = mass_dat(region)%rxm_t(1:imr,1:jmr,l,tracer_loc(itr))
                paty = mass_dat(region)%rym_t(1:imr,1:jmr,l,tracer_loc(itr))
                patz = mass_dat(region)%rzm_t(1:imr,1:jmr,l,tracer_loc(itr))
                filled = .true.
              end if
            case default
              write (gol,'("unsupported which_par : ",a)') trim(which_par); call goErr
              write (gol,'("in ",a)') rname; call goErr; status=1; return
          end select
        else if ( itr <= ntrace ) then
          l_base = 0
          if (myid>0) l_base = sum(lmar(0:myid-1))
          l_loc = l - l_base
          if ( (l_loc >= 1) .and. (l_loc <= lmloc) ) then
            pat  = 0.0 !chem_dat(region)%rm_k(1:imr,1:jmr,l_loc,itr)    ! ntracet+1:ntrace
            patx = 0.0
            paty = 0.0
            patz = 0.0
            filled = .true.
          end if
        end if
        
        if ( filled ) then

          !write (gol,'(a,i3," ",a,i3,3es14.4)') key, itr, names(itr), l, maxval(pat), sum(pat)/size(pat), pat(3,3); call goBug

          write (gol,*) key, itr, names(itr), l, sum(pat), sum(patx), sum(paty), sum(patz); call goBug

!          do j = 1, jmr
!            do i = 1, imr
!              if (i/=4) cycle
!              if (j/=4) cycle
!              write (gol,'(a,i3," ",a,3i4,es14.4,f8.2)') key, itr, names(itr), i,j,l, pat(i,j), log10(pat(i,j)); call goBug
!            end do
!          end do

        end if
        
        call par_barrier
        
      end do
  
      call par_barrier
        
    end do
    
    deallocate( pat )

    ! ok
    status = 0
    
  end subroutine tracer_print


end module tracer_data


